/*
 * AddInInfo.h
 *
 * Author: J. Steingraeber
 *
 * Date: 2001/02/27
 *
 * Changes and optimizations by Moritz Beutel, 2006/06/05
 *
 * Released to the public domain
 */


#ifndef AddInInfo_h_included
#define AddInInfo_h_included

#define MAXCOMMENTLENGTH 64

/*
 * AddInInfo
 *
 * structure to receive Add-In data of a specific Add-In
 *
 * this structure is filled from the header of an Add-In
 * by GetAddInInfo
 *
 */
typedef struct tagAddInInfo
{
  char name[16];                /* zero terminated string */
  unsigned char mode;           /* Add-In mode */
  unsigned char submode;        /* Add-In submode */
  unsigned long size;           /* length in bytes */
  unsigned long orgsize;        /* original length */
  unsigned long address;        /* linear address of Add-In (for StartAddIn) */
  unsigned int flags;           /* extra information: deleted, "packed" */
  char version[16];             /* date and time of creation and version */               /* EXT */
  char lib_version[16];         /* date, time and version of library compiled against */  /* EXT */
  unsigned char menuicon[176];  /* ready for LibPutGraph() data 45*28 pixels */           /* EXT */
  unsigned char listicon[84];   /* ready for LibPutGraph() data 27*20 pixels */           /* EXT */
  char comment[64];                                                                       /* EXT */
} AddInInfo; /* 318 bytes */


/*
 * flags for AddInInfo.flags
 */
#define ADDININFO_DELETED 1 /* "packed" Add-Ins are always marked as deleted */
#define ADDININFO_PACKED  2
#define ADDININFO_PACK    4


/*
 * flags for ext parameter
 */
#define ADDININFO_EXT_NONE      0
#define ADDININFO_EXT_VERSION   1
#define ADDININFO_EXT_MENUICON  2
#define ADDININFO_EXT_LISTICON  4




/*
 * SwitchBank
 *
 * makes RAM or FLASH memory accessible at certain bank
 *
 * addr A31-A16 of source address (must be even)
 *      use 0000 - 00FE for RAM (only 0000 will work)
 *      use 0100 - 010E for program flash (1st MB, PVOS)
 *      use 0110 - 011E for program flash (2nd MB, Add-In)
 *      use 0120 - 013E for data flash (4MB data)
 *
 * bank destination bank
 *      use 0 for address space 0000:0000 (forbidden)
 *      use 1 for address space 2000:0000
 *      use 2 for address space 4000:0000
 *      use 3 for address space 6000:0000
 *      use 4 for address space 8000:0000 (forbidden)
 *      use 5 for address space A000:0000
 *      use 6 for address space C000:0000 (forbidden)
 *
 * remarks:
 *   you can not switch bank 0, 4 and 6, because that will crash your
 *   PV; bank 0 contains the interrupt vectors, and every program
 *   expects to find RAM here; bank 4 contains your Add-In, so
 *   switching this would make your Add-In vanishing; bank 6 contains
 *   the BIOS, your PV can not operate without it
 *
 * examples:
 *
 *   SwitchBank(0x0000,3);
 *     will make 128kB RAM accessible at address 6000:0000 so RAM can
 *     be accessed at 0000:0000 and 6000:0000 in the same way (which
 *     is possible but stupid, you still have only 128kB of RAM)
 *
 *   SwitchBank(0x0104,5);
 *     will make Fonts accessible at address A000:0000 and Graphics
 *     accessible at B000:0000
 *
 *   SwitchBank(0x0120,1);
 *     will make first 128kB of data flash accessible at 2000:0000
 *
 */
void _asm_switchbank(char *,int,int);
#define SwitchBank(addr,bank) _asm_switchbank("\n MOV BH,AL\n " \
                                                 "MOV AL,AH\n " \
                                                 "MOV AH,BL\n " \
                                                 "OR AH,80h\n " \
                                                 "INT 0C8h\n "  \
                                                 "MOV AL,BH\n " \
                                                 "MOV AH,BL\n " \
                                                 "INT 0C8h\n",addr,bank);


/*
 * GetAddInInfo
 *
 * fills an AddInInfo structure with information from
 * requested Add-In iNr
 *
 * iNr is of range 0..63 with regular Add-Ins at 0,4,8,...,60
 * and packed Add-Ins possibly in between
 *
 * returns 1 if Add-In is found and AddInInfo valid
 * returns 0 otherwise, e.g. there is no Add-In starting at the slot
 *
 * The strategy to look for regular Add-Ins is to call GetAddInInfo
 * for slot 0,4,8,...,60 and check the flags if the Add-In is deleted
 *
 * Example:
 *
 * int i;
 * int k, maxsub; // needed for "packed" Add-In search only
 * AddInInfo info;
 *
 * for (i=0;i/lt;64;i+=4)
 * {
 *     if (GetAddInInfo(i,&info) && ((info.flags & ADDININFO_DELETED) == 0))
 *     {
 *         // regular not deleted Add-In found
 *         // ...
 *     }
 * }
 *
 * The strategy to look for "packed" Add-Ins is to examine the
 * size of each not deleted regular Add-In found and test for
 * the next (size / 0x4000) Add-Ins
 *
 * Example:
 *
 * // continuation of above example if regular Add-In was found
 *
 * maxsub = info.size / 0x4000;
 *
 * for (k=1;k/lt;maxsub;++k)
 * {
 *     if (GetAddInInfo(i+k,&info))
 *     {
 *         // "packed" Add-In found
 *         // ADDININFO_DELETED is always set
 *     }
 * }
 *
 */



int GetAddInInfo (int iNr, AddInInfo* pInfo, int ext);


/*
 * StartAddIn
 *
 * calls the Add-In at address lLinAddr
 *
 * sets some PVOS internal variables so that PV can savely switch off and on
 *
 * lLinAddr should be taken from AddInInfo
 */
void StartAddIn (unsigned long lLinAddr);


/*
 * CanDelAddIn
 *
 * test if the Add-In number iNr (0..63) could be deleted
 *
 * not deletable are Add-Ins which
 * - belong to PVOS (Mail or PocketSheet)
 * - are "packed"
 * - are currently running
 *
 * Returns 1 if the Add-In could be deleted by DelAddIn()
 * returns 0 otherwise
 */
int CanDelAddIn (int iNr);


/*
 * DelAddIn
 *
 * deletes the Add-In number iNr (0..63)
 *
 * actually this will only mark the Add-In as deleted
 * as it is done by PVOS/PVM
 *
 * some precautions take place to not delete build-in Add-Ins
 * on PV-S[24]50 or PV-750(+)
 *
 * Returns 1 if the Add-In is successfully marked as deleted,
 * returns 0 otherwise
 */
int DelAddIn (int iNr);

/*
 * DefragAddInMem
 *
 * defragments Add-In memory
 *
 * does not return -> LibJmpMenu()
 *
 */
void DefragAddInMem (void);


/*
 * CorrectMenuMaxPage()
 *
 * call this after each change of the number of installed Add-Ins
 * (e.g. after delting one)
 *
 */
void CorrectMenuMaxPage (void);

#endif

