#ifndef _UCL_FILE_HPP
#define _UCL_FILE_HPP


#include <string>
#include <vector>
#include <cstdlib>


namespace ucl
{

std::string extractFileName (const std::string& path);
std::string extractFilePath (const std::string& path);
std::string extractFileExt  (const std::string& path);


class BasicTextFile
{
protected:
    BasicTextFile (void) {} /* direct construction forbidden, only for derivation */

    typedef std::vector <std::string> stringlist_t;
    static stringlist_t::iterator getIteratorFromIndex (stringlist_t& sl, unsigned int idx);
    static bool BasicTextFile::read_string (std::FILE* stream, std::string& str);
	virtual bool load_internal (const std::string& name) const = 0;

public:
	bool loadFromFile (const std::string& name);
    virtual bool saveToFile (const std::string& name) const = 0;

    virtual void clear (void) = 0;

    virtual ~BasicTextFile (void) {};
};


class TextFile : public BasicTextFile
{
protected:
	mutable stringlist_t lines;
	virtual bool load_internal (const std::string& name) const;
public:
    TextFile (void) {}
	TextFile (const std::string& name);
    TextFile (const TextFile& tf)
     : lines (tf.lines)
    { }
    virtual ~TextFile (void) {}

    void operator = (const TextFile& tf) { lines = tf.lines; }

    std::string& operator [] (unsigned int idx);
    const std::string& operator [] (unsigned int idx) const;

    bool loadFromFile (const std::string& name);
    virtual bool saveToFile (const std::string& name) const;

    virtual void clear (void);

    unsigned int lineCount (void) const;

    void addLine    (const std::string& text);
    void insertLine (unsigned int line, const std::string& text);
	void deleteLine (unsigned int line);
};


class IniFile : public BasicTextFile
{
public:
	class Section;


private:
	class StringProxy;

	friend class Section;
	friend class StringProxy;

	static std::string extractText (const std::string& str);
	static std::string extractVar (const std::string& str);

	class StringProxy
	{
		friend class Section; /* Section may construct StringProxy objects */

	private:
		std::string& str;
		StringProxy (const StringProxy& s) : str (s.str)
		{ }
		StringProxy (std::string& s)
		 : str (s)
		{ }

	public:
		~StringProxy (void) {}
		const StringProxy& operator = (const StringProxy& s) { str = extractVar (str) + '=' + extractText (s.str); return (s); }
		const std::string& operator = (const std::string& s) { str = extractVar (str) + '=' + s; return (str); }
		const std::string& operator += (const std::string& s) { str += s; return (str); }
		operator std::string (void) const { return (extractText (str)); }
		std::string asString (void) const { return (extractText (str)); }
		int asInteger (void) const { return (std::atoi (extractText (str).c_str ())); }
	};

public:
	class Section : public TextFile
	{
		friend class IniFile;

	private:
		Section (void) {}
		Section (const std::string& nm)
		 : name (nm)
		{ }

    public:
        Section (const Section& sf)
         : name (sf.name), TextFile (sf)
        { }

		std::string name; /* YES, PUBLIC - why not allow changing the name? */

		void operator = (const Section& sf) { lines = sf.lines; name = sf.name; }

		virtual void clear (void) { TextFile::clear (); }

		std::string& operator [] (unsigned int idx) { return (TextFile::operator [] (idx)); }
		const std::string& operator [] (unsigned int idx) const { return (TextFile::operator [] (idx)); }
  
		StringProxy operator [] (const std::string& var);
		const StringProxy operator [] (const std::string& var) const;

		bool entryExists (const std::string& entry) const;
	};

private:
	mutable std::vector <Section> sections;
	virtual bool load_internal (const std::string& name) const;

public:
	IniFile (void) {}
	IniFile (const std::string& name);
	IniFile (const IniFile& ifl);

	virtual ~IniFile (void) {}

	Section& operator [] (const std::string& section);
	const Section& operator [] (const std::string& section) const;

	void addSection (const std::string& section);
	bool sectionExists (const std::string& section) const;

	virtual bool saveToFile (const std::string& name) const;

	virtual void clear (void);
};




    /* inline implementations */
inline bool BasicTextFile::loadFromFile (const std::string& name)
    { return (load_internal (name)); }
inline TextFile::TextFile (const std::string& name)
	{ load_internal (name); }
inline unsigned int TextFile::lineCount (void) const
    { return (lines.size ()); }
inline IniFile::IniFile (const std::string& name)
	{ load_internal (name); }
inline IniFile::IniFile (const IniFile& ifl)
	{ sections = ifl.sections; }






} /* end namespace ucl */

#endif
